<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */
/*
 * Copyright 2004-2007 Project Guarana Development Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * @package ficus.parameters.datatypes
 */
/**
 * @file AnyURIParameter.php
 * @brief AnyURIParameter for php
 * @author <a href="mailto:kent@guarana.cc">ISHITOYA Kentaro</a>
 * @author <a href="mailto:sumi@wakhok.ac.jp">SUMI Masafumi</a>
 * @version $Id: AnyURIParameter.php 2 2007-07-11 10:37:48Z ishitoya $
 * 
 * ValidatableSimpleParameter with normailizer.
 * treat param as anyURI
 */

require_once("ficus/exception/ValidationException.php");
require_once("ficus/net/URI.php");
require_once("ficus/net/URIBuilder.php");
require_once("ficus/parameters/ValidatableSimpleParameter.php");

/**
 * @class Ficus_AnyURIParameter
 */
class Ficus_AnyURIParameter extends Ficus_ValidatableSimpleParameter
{
	const TYPENAME = "anyURI";
	
	/**
 	 * construct parameter with name, value, and specific validator.
 	 * @param $name string name of this parameter.
 	 * @param $value string value of this parameter.
 	 * @param $validator array array of Ficus_Validator.
 	 */
 	public function __construct($name, $value = null, $validator = array()){
		parent::__construct($name, self::TYPENAME, $value);
		$this->setValidator($validator);
	}

	/**
	 * check param that associated with simple parameter
	 * @param $parameter array data
	 * @return array validated param
     * @throw Ficus_ValidationException invalidate parameter.
	 */
	public function validate($parameter){
		$value = trim($parameter["value"]);

        try {
            $uri = Ficus_URIBuilder::create($value);
        } catch (Ficus_URISyntaxException $e) {
            $uri = null;
        }

		if(empty($value) || $uri){
			parent::validate($parameter);
		}else{
			throw new Ficus_ValidationException("parameter {$this->name()} is not valid URI");
		}
	}
}
?>
